function Level()
{
	this.tracks = new Array();
	this.parts = new Array();
	this.length = 0;
	this.time = 0;

	this.update = function(time)
	{
		if((time > this.length) || (time < 0)){
			return updateNoneTrack();
		}

		for(var i=0;i<this.tracks.length;i++)
		{
			if((this.tracks[i].pos <= time) && (this.tracks[i].pos + this.tracks[i].length > time)){
				return this.tracks[i].update(time);
			}
		}

		return updateNoneTrack();
	}

	this.updateParts = function(time)
	{
		for(var i=0;i<this.parts.length;i++)
		{
			this.parts[i].update(time);
		}
		this.time += time;
	}

	this.draw = function()
	{
		for(var j=0;j<this.parts.length;j++)
		{
			this.parts[j].draw();
		}

		for(var i=0;i<this.tracks.length;i++){
			this.tracks[i].draw();
		}

		drawPortal(this.length+1,this.time);
	}

	this.left = false;

	this.add = function(type,length,nrots)
	{
		if(type >= 10){
			if(length > this.length){
				this.parts.push(new ParticalEmiter(new Vector(0,0,length),true));
				return;
			}
			for(var i=0;i<this.tracks.length;i++)
			{
				if((this.tracks[i].pos <= length) && (this.tracks[i].pos + this.tracks[i].length > length)){
					var t = true;
					if(this.tracks[i].type == ttd){
						t = false;
					}
					this.parts.push(new ParticalEmiter(this.getSafePos(length,this.left),t));
					this.left = !this.left;
					return;
				}
			}
		}
		this.tracks.push(new Track(type,this.length,length,nrots));
		this.length += length;
	}

	this.getSafePos = function(length,left)
	{
		if((length > this.length) || (length < 0)){
			return new Vector(0,0,length);
		}
		for(var i=0;i<this.tracks.length;i++)
		{
			if((this.tracks[i].pos <= length) && (this.tracks[i].pos + this.tracks[i].length > length)){
				return this.tracks[i].safe(length,left);
			}
		}
	}

	this.findNextObs = function(t){
		var m = this.length;
		for(var i=0;i<this.tracks.length;i++)
		{
			if((this.tracks[i].pos <= t) && (this.tracks[i].pos + this.tracks[i].length > t)){
				if(this.tracks[i].type == trackType_none){
					return t;
				}
				if(this.tracks[i].type == trackType_double){
					m = this.tracks[i].pos + this.tracks[i].length;
					break;
				}
				i++;
				while(i<this.tracks.length){
					if((this.tracks[i].type == trackType_double)||(this.tracks[i].type == trackType_none)){
						m = this.tracks[i].pos;
						break;
					}
					i++;
				}
				break;
			}
		}
		for(var j=0;j<this.parts.length;j++)
		{
			if(this.parts[j].pos.z > t){
				if(this.parts[j].pos.z < m){
					return this.parts[j].pos.z;
				}
			}
		}
		return m;
	}

	this.findPrevObs = function(t){
		var m = 0;
		for(var i=0;i<this.tracks.length;i++)
		{
			if((this.tracks[i].pos <= t) && (this.tracks[i].pos + this.tracks[i].length > t)){
				if(this.tracks[i].type == trackType_none){
					return t;
				}
				if(this.tracks[i].type == trackType_double){
					m = this.tracks[i].pos;
					break;
				}
				i--;
				while(i>=0){
					if((this.tracks[i].type == trackType_double)||(this.tracks[i].type == trackType_none)){
						m = this.tracks[i].pos+this.tracks[i].length;
						break;
					}
					i--;
				}
				break;
			}
		}
		for(var j=0;j<this.parts.length;j++)
		{
			if(this.parts[j].pos.z < t){
				if(this.parts[j].pos.z > m){
					m = this.parts[j].pos.z;
				}
			}
		}
		return m;
	}

	this.findSafePos = function(t)
	{
		var i = t;
		while(i>0)
		{
			var p = this.findPrevObs(i);
			var n = this.findNextObs(i);
			if(Math.abs(n-p) >= safeLength)
			{
				return n-safeLength+1//(n+p)/2;
			}
			i = p-1;
		}
		return 0;
	}

	this.playerHitPart = function()
	{
		for(var j=0;j<this.parts.length;j++)
		{
			var v = new Vector(0,0,playerPos.z);
			v = subVector(v,this.parts[j].pos);
			v.y = playerUp;
			if(!this.parts[j].center){
				v.x = playerPos.x - this.parts[j].pos.x;
			}else{
				v.x = 0;
			}
			if(dotVector(v,v) < particalSize*particalSize){
				return true;
			}
		}
		return false;
	}

	return this;
}

var tp = 10;

function loadLevel(l){
	delete levelData;
	genLevel(l);
	//runScript("levels/l"+l+".js");
}