var trackType_single = 0;
var trackType_double = 1;
var trackType_ring = 2;
var trackType_invring = 3;
var trackType_none = 4;

var ttn = trackType_none;
var tts = trackType_single;
var ttd = trackType_double;
var ttr = trackType_ring;
var tti = trackType_invring;

function drawSingleTrack()
{
	set3DLineWidth(0.15);
	setColor(0,0,1);
	for(var i=this.pos;i<this.pos+this.length;i++)
	{
		draw3DLine(new Vector(0,0,i),
			new Vector(0,0,i+1));
	}
}

function drawDoubleTrack()
{
	set3DLineWidth(0.15);
	setColor(1,0,1);
	for(var i=this.pos;i<this.pos+this.length;i++)
	{
		draw3DLine(new Vector(-0.25,0,i),
			new Vector(-0.25,0,i+1));
		draw3DLine(new Vector(0.25,0,i),
			new Vector(0.25,0,i+1));
	}
}

function drawRingTrack()
{
	set3DLineWidth(0.15);
	setColor(1,0,0);
	draw3DLine(new Vector(0,0,this.pos),
		new Vector(0,0,this.pos+1));
	draw3DLine(new Vector(0,0,this.pos+this.length-1),
		new Vector(0,0,this.pos+this.length));
	for(var i=this.pos;i<this.pos+this.length-2;i++)
	{
		var t = (i-this.pos)/(this.length-2);
		var t2 = (i-this.pos+1)/(this.length-2);
		draw3DLine(new Vector(Math.sin(2*Math.PI*t*this.nrots),Math.cos(Math.PI+2*Math.PI*t*this.nrots)+1,i+1),
			new Vector(Math.sin(2*Math.PI*t2*this.nrots),Math.cos(Math.PI+2*Math.PI*t2*this.nrots)+1,i+2));
	}
}

function drawInvRingTrack()
{
	set3DLineWidth(0.15);
	setColor(0,1,0);
	draw3DLine(new Vector(0,0,this.pos),
		new Vector(0,0,this.pos+1));
	draw3DLine(new Vector(0,0,this.pos+this.length-1),
		new Vector(0,0,this.pos+this.length));
	for(var i=this.pos;i<this.pos+this.length-2;i++)
	{
		var t = (i-this.pos)/(this.length-2);
		var t2 = (i-this.pos+1)/(this.length-2);
		draw3DLine(new Vector(-Math.sin(2*Math.PI*t*this.nrots),Math.cos(Math.PI+2*Math.PI*t*this.nrots)+1,i+1),
			new Vector(-Math.sin(2*Math.PI*t2*this.nrots),Math.cos(Math.PI+2*Math.PI*t2*this.nrots)+1,i+2));
	}
}

function playerLandOnTrack(){
	if(playerCanJump && (playerUp < 0)){
		playerUp = 0;
		playerVel = 0;
	}
}

function trackSetPlayerJump(x){
	if(Math.abs(playerPos.x-x)<trackLineLandingWidth){
		playerCanJump = true;
		return true;
	}
	playerCanJump = false;
	return false;
}

function updateSingleTrack(time)
{
	trackSetPlayerJump(0);

	playerLandOnTrack();

	playerPos.x = 0;
	playerPos.y = playerUp;

	return true;
}

function updateDoubleTrack(time)
{
	if(!trackSetPlayerJump(-doubleTrackWidth/2)){
		trackSetPlayerJump(doubleTrackWidth/2);
	}

	playerLandOnTrack();

	if(playerLeft){
		playerPos.x = -doubleTrackWidth/2;
	}else{
		playerPos.x = doubleTrackWidth/2;
	}
	playerPos.y = playerUp;

	return true;
}

function updateRingTrack(time)
{

	if(((time-this.pos)<1)||((this.pos+this.length-time)<1)){
		trackSetPlayerJump(0);

		playerLandOnTrack();

		playerPos.x = 0;
		playerPos.y = playerUp;

		if((this.pos+this.length-time)<1){
			return false;
		}
	}else{
		playerCanJump = true;
		
		playerLandOnTrack();

		var t = (time-this.pos-1)/(this.length-2);
		playerPos.x = Math.sin(2*Math.PI*t*this.nrots)-Math.sin(2*Math.PI*t*this.nrots)*playerUp;
		playerPos.y = Math.cos(Math.PI+2*Math.PI*t*this.nrots)+1-Math.cos(Math.PI+2*Math.PI*t*this.nrots)*playerUp;

		return false;
	}

	return true;
}

function updateInvRingTrack(time)
{
	if(((time-this.pos)<1)||((this.pos+this.length-time)<1)){
		trackSetPlayerJump(0);

		playerLandOnTrack();

		playerPos.x = 0;
		playerPos.y = playerUp;

		if((this.pos+this.length-time)<1){
			return false;
		}
	}else{
		playerCanJump = true;
		
		playerLandOnTrack();

		var t = (time-this.pos-1)/(this.length-2);
		playerPos.x = -Math.sin(2*Math.PI*t*this.nrots)+Math.sin(2*Math.PI*t*this.nrots)*playerUp;
		playerPos.y = Math.cos(Math.PI+2*Math.PI*t*this.nrots)+1-Math.cos(Math.PI+2*Math.PI*t*this.nrots)*playerUp;

		return false;
	}

	return true;
}

function updateNoneTrack(time)
{
	playerCanJump = false;

	playerPos.y = playerUp;

	return false;
}

function getSingleTrackSavePos(time)
{
	return new Vector(0,0,time);
}

function getDoubleTrackSavePos(time,l)
{
	var x = doubleTrackWidth/2;
	if(l){
		x = -doubleTrackWidth/2;
	}
	return new Vector(x,0,time);
}

function getRingTrackSavePos(time)
{
	if(((time-this.pos)<1)||((this.pos+this.length-time)<1)){
		return new Vector(0,0,time);
	}
	var t = (time-this.pos-1)/(this.length-2);
	return new Vector(Math.sin(2*Math.PI*t*this.nrots),
		Math.cos(Math.PI+2*Math.PI*t*this.nrots)+1,
		time);
}

function getInvRingTrackSavePos(time)
{
	if(((time-this.pos)<1)||((this.pos+this.length-time)<1)){
		return new Vector(0,0,time);
	}
	var t = (time-this.pos-1)/(this.length-2);
	return new Vector(-Math.sin(2*Math.PI*t*this.nrots),
		Math.cos(Math.PI+2*Math.PI*t*this.nrots)+1,
		time);
}

function Track(type,pos,length,nrots)
{
	this.type = type;
	this.pos = pos;
	this.length = length;

	switch(type)
	{
	case trackType_single:
		this.draw = drawSingleTrack;
		this.update = updateSingleTrack;
		this.safe = getSingleTrackSavePos;
		break;
	case trackType_double:
		this.draw = drawDoubleTrack;
		this.update = updateDoubleTrack;
		this.safe = getDoubleTrackSavePos;
		break;
	case trackType_ring:
		this.draw = drawRingTrack;
		this.update = updateRingTrack;
		this.safe = getRingTrackSavePos;
		this.nrots = nrots;
		break;
	case trackType_invring:
		this.draw = drawInvRingTrack;
		this.update = updateInvRingTrack;
		this.safe = getInvRingTrackSavePos;
		this.nrots = nrots;
		break;
	case trackType_none:
		this.draw = function(){};
		this.update = updateNoneTrack;
		this.safe = getSingleTrackSavePos;
		break;
	}

	return this;
}